import datetime


def set_cookie(response, key, value, days_expire=90):
    if days_expire is None:
        #one year
        max_age = 365 * 24 * 60 * 60
    else:
        max_age = days_expire * 24 * 60 * 60
    expires = datetime.datetime.strftime(datetime.datetime.utcnow() +
                                         datetime.timedelta(seconds=max_age),
                                         "%a, %d-%b-%Y %H:%M:%S GMT")
    response.set_cookie(key, value, max_age=max_age, expires=expires)



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.db.models.manager import Manager
from django.core.exceptions import ValidationError


POLLTYPE_CHOICES = (
        ('0', 'Single'),
        ('1', 'Multiple'),
    )


class PublishManager(Manager):
    def get_query_set(self):
        return super(PublishManager, self).get_query_set().filter(publish=True)


class Poll(models.Model):
    title = models.CharField(max_length=250,
                             verbose_name=_('Title'),
                             help_text=_('The parameter is used as question to user'))
    queue = models.ForeignKey('Queue',
                              blank=True,
                              null=True,
                              verbose_name=_('Queue'),
                              help_text=_('Do you want to use the poll as a stand alone poll or insert it into the queue?'))
    polltype = models.CharField(max_length=1,
                                choices=POLLTYPE_CHOICES,
                                verbose_name=_('Poll type'),
                                help_text=_('Choose the poll type'))
    startdate = models.DateField(verbose_name=_('Start date'),
                                 help_text=_('Must be unique'))
    publish = models.BooleanField(default=True, verbose_name=_('Publish'))
    votes = models.ManyToManyField('Vote',
                                   related_name='%(app_label)s_%(class)s_related',
                                   blank=True,
                                   verbose_name=_('Votes'),
                                   help_text=_('Choose variants of answers'))

    objects = models.Manager()
    publish_manager = PublishManager()

    def clean(self):
        if self.queue is not None:
            err_msg = _('"Start dates" of polls in the same queue must be different')
            objs = Poll.objects.filter(queue=self.queue,
                                       startdate=self.startdate)
            count = objs.count()

            if count > 1:
                raise ValidationError(err_msg)
            elif count == 1:
                if objs[0] != self:
                    raise ValidationError(err_msg)

    def get_vote_count(self):
        return Vote.objects.filter(poll=self).count()

    vote_count = property(fget=get_vote_count)

    def get_cookie_name(self):
        return str('poll_%s' % (self.pk))

    def __unicode__(self):
        return self.title

    def print_polltype(self):
        return POLLTYPE_CHOICES[int(self.polltype)][1]

    class Meta:
        ordering = ['-startdate']
        verbose_name = _('Poll')
        verbose_name_plural = _('Polls')


class Queue(models.Model):
    title = models.CharField(unique=True,
                             max_length=250,
                             verbose_name=_('Queue name'),
                             help_text=_('It must be unique'))
    auth = models.BooleanField(verbose_name=_('Need auth?'),
                               help_text=_('Do the poll queue is for authenticated users only or not? (If yes, users must have "can_vote" permission to vote)'))

    def __unicode__(self):
        if self.auth:
            auth = _('With auth')
        else:
            auth = _('Without auth')

        return u'%s (%s)' % (self.title, auth)

    class Meta:
        ordering = ['-title']
        verbose_name = _('Queue')
        verbose_name_plural = _('Queues')


class Item(models.Model):
    poll = models.ForeignKey(Poll)
    userbox = models.BooleanField(verbose_name=_('Its userbox?'),
                                  help_text=_('Set it, if you want user to type its own answer. (show as textbox)'))
    value = models.CharField(max_length=250,
                             verbose_name=_('Value'),
                             help_text=_('Its a title of item'))
    index = models.SmallIntegerField(default='0',
                                     verbose_name=_('Position'),
                                     help_text=_('Its for positioning only'))

    def __unicode__(self):
        return u'%s (%d)' % (self.value, self.index)

    class Meta:
        ordering = ['index']


class Vote(models.Model):
    poll = models.ForeignKey(Poll, verbose_name=_('Poll'))
    ip = models.IPAddressField(verbose_name=_('User\'s IP'))
    user = models.ForeignKey(User,
                             blank=True,
                             null=True,
                             verbose_name=_('User'))
    datetime = models.DateTimeField(auto_now_add=True)
    choices = models.ManyToManyField(Item,
                                     through='Choice',
                                     verbose_name=_('Voited items'))

    def __unicode__(self):
        if isinstance(self.user, User):
            return self.user.username
        return self.ip

    class Meta:
        verbose_name = _('Vote')
        verbose_name_plural = _('Votes')


class Choice(models.Model):
    vote = models.ForeignKey(Vote)
    item = models.ForeignKey(Item)
    uservalue = models.CharField(max_length=250, blank=True, null=True)



from django.http import HttpResponse
from poll.models import Poll, Item, Vote, Choice
from django.db import transaction
from django.utils import simplejson
from utils import set_cookie


def authpass(user, queue):
    if queue != None:
        if queue.auth:
            if not user.is_authenticated():
                return False
    return True


#TODO: Need to optimize
@transaction.commit_on_success
def poll_ajax_vote(request, poll_pk):
    if request.is_ajax():
        try:
            poll = Poll.objects.get(pk=poll_pk)
            if poll.queue:
                if not authpass(request.user, poll.queue):
                    return HttpResponse('Non-authenticated users can\'t vote',
                                        status=400)

            chosen_items = simplejson.loads(request.GET['chosen_items'])
        except:
            return HttpResponse('Wrong parameters', status=400)

        if request.user.is_authenticated():
            user = request.user
        else:
            user = None

        vote = Vote.objects.create(poll=poll,
                                   ip=request.META['REMOTE_ADDR'],
                                   user=user)
        try:
            for item_pk, value in chosen_items.items():
                item = Item.objects.get(pk=item_pk)

                if item.userbox:
                    Choice.objects.create(vote=vote, item=item,
                                          uservalue=value)
                else:
                    Choice.objects.create(vote=vote, item=item)
        except:
            return HttpResponse('Data recognition failed', status=400)

        response = HttpResponse(status=200)
        set_cookie(response, poll.get_cookie_name(), poll_pk)

        return response

    return HttpResponse(status=400)


def poll_ajax_result(request, poll_pk):
    if request.is_ajax():
        try:
            poll = Poll.objects.get(pk=poll_pk)
        except:
            return HttpResponse('Wrong parameters', status=400)

        #Send data for results
        data = {}

        for item in Item.objects.filter(poll=poll):
            subdata = {
                       'index': item.index,
                       'title': item.value,
                       'count': Choice.objects.filter(item=item).count(),
                       }

            data[item.pk] = subdata

        data['total'] = Vote.objects.filter(poll=poll).count()

        return HttpResponse(simplejson.dumps(data))
    return HttpResponse(status=400)



from django.contrib import admin
from poll.models import Poll, Item, Queue, Vote, Choice
from django.utils.translation import ugettext_lazy as _


class PollItemInline(admin.TabularInline):
    model = Item
    extra = 5
    max_num = 10


class PollAdmin(admin.ModelAdmin):
    list_display = ('title', 'queue', 'startdate', 'polltype', 'vote_count',
                    'publish')
    inlines = [
        PollItemInline,
    ]

    fieldsets = (
                 (None, {'fields': ('title',)}),
                 (_('Options'), {'fields': ('publish', 'polltype', 'queue',
                                            'startdate',)}),
                 )


class VoteChoiceItemInline(admin.TabularInline):
    model = Choice
    extra = 5
    max_num = 10
    readonly_fields = ('item', 'uservalue')


class VoteAdmin(admin.ModelAdmin):
    list_display = ('poll', 'ip', 'user', 'datetime')
    list_filter = ('poll', 'datetime')
    inlines = [
        VoteChoiceItemInline,
    ]

admin.site.register(Poll, PollAdmin)
admin.site.register(Queue, admin.ModelAdmin)
admin.site.register(Vote, VoteAdmin)



from django.conf.urls.defaults import patterns, url
from poll.ajax import poll_ajax_vote, poll_ajax_result

urlpatterns = patterns('',
    url(r'^vote/(?P<poll_pk>\d)$',
        poll_ajax_vote,
        name='poll_ajax_vote'),
    url(r'^result/(?P<poll_pk>\d)$',
        poll_ajax_result,
        name='poll_ajax_result'),
)



from django import template
from poll.models import Poll, Item, Queue
from django.conf import settings
from django.utils.safestring import SafeUnicode
from django.utils.datetime_safe import datetime
from poll.ajax import authpass


register = template.Library()


@register.inclusion_tag('polls.html', takes_context=True)
def poll(context, poll):
    can_vote = True
    if poll.queue:
        can_vote = authpass(context['user'], poll.queue)
    return {'poll': poll,
            'poll_type': poll.print_polltype(),
            'items': Item.objects.filter(poll=poll),
            'user': context['user'],
            'can_vote': can_vote,
            'request': context['request'],
            'STATIC_URL': settings.STATIC_URL}


@register.inclusion_tag('polls.html', takes_context=True)
def poll_queue(context, queue):
    try:
        if isinstance(queue, SafeUnicode):
            tmp_queue = Queue.objects.get(title=queue)
        else:
            tmp_queue = Queue.objects.get(queue)
    except:
        raise Exception('Queue not found')

    tmp_polls = Poll.publish_manager.filter(queue=tmp_queue,
                                            startdate__lte=datetime.now())

    if len(tmp_polls) > 0:
        cur_poll = tmp_polls[0]
    else:
        cur_poll = None

    return poll(context, cur_poll)


class RenderItemsClass(template.Node):
    def __init__(self, poll, items):
        self.poll = template.Variable(poll)
        self.items = template.Variable(items)

    def render(self, context):
        poll = self.poll.resolve(context)
        items = self.items.resolve(context)
        #'name' = item.pk
        pattern1 = u'{3}<br /><input name="poll_{0}" ' \
                   'type="{1}" id="{2}" value="" /><br />'
        pattern2 = u'<input name="poll_{0}" type="{1}" id="{2}" /> {3}<br />'
        result = u''

        #Choose an input type
        for item in items:
            if item.userbox:
                input_type = 'textbox'
                pattern = pattern1
            else:
                poll_type = poll.print_polltype()

                if poll_type == 'Single':
                    input_type = 'radio'
                elif poll_type == 'Multiple':
                    input_type = 'checkbox'
                pattern = pattern2

            result += pattern.format(poll.pk, input_type, item.pk, item.value)

        return result


@register.tag
def render_items(parser, token):
    tag, poll, items = token.split_contents()
    return RenderItemsClass(poll, items)



from distutils.core import setup

setup(name='django-poll-system',
      version='0.2.1',
      description='The application to organize polling (or voting) on your site.',
      author='RafRaf',
      maintainer='RafRaf',
      author_email='smartrafraf@gmail.com',
      url='http://polltest.pythonism.ru',
      download_url='https://bitbucket.org/RafRaf/django-poll-system',
      packages=['poll', 'poll.templatetags'],
      package_data={'poll': ['locale/ru/LC_MESSAGES/*', 'templates/*']},
     )